import intfifo.FIFO;
import intfifo.EmptyQueueException;
import intfifo.FullQueueException;

public class FIFOWrapper {

  private final FIFO f;
  private final int max;
  private int size;

  FIFOWrapper (FIFO f, int max) {
    this.f = f;
    this.max = max;
    this.size = 0;
  }

  private void fail (String msg) {
    TestFIFO.fail(msg);
  }
  
  @Override
  public String toString () {
    return f.toString();
  }

  public int size () {
    int s = f.size();
    if (this.size != s)
      fail("size mismatch: should be " + this.size + " but is " + s);
    return size;
  }

  public void get (int expected) {
    int x;
    boolean exceptionExpected = (size() == 0);
    try {
      x = f.get();
      size--;
      if (exceptionExpected)
        fail("get() was expected to fail, but succeeded");
      if (x != expected)
        fail("get() was expected to return " + expected + ", but returned " + x);
    } 
	catch (EmptyQueueException eqe){
      if (!exceptionExpected)
        fail("get() was expected to succeed, but threw " + eqe);
	}
	catch (Exception e) {
      fail("get() threw " + e);
    }
    size();
  }

  public void put (int n) {
    boolean exceptionExpected = (size() == max);
    try {
      f.put(n);
      size++;
      if (exceptionExpected)
        fail("put(n) was expected to fail, but succeeded");
    } 
	catch (FullQueueException fqe){
      if (!exceptionExpected)
        fail("put(n) was expected to succeed, but threw " + fqe);
	}
	catch (Exception e) {
	  fail("put(n) threw " + e);
    }      
    size();
  }

}

