import intsort.*;

import java.util.*;

class SortError extends Error {}

public class TestSort {

    public static void fail (String msg) {
        System.err.println(msg);
        throw new SortError();
    }

    public static boolean checkOrdered(int[] a) {
        for (int i = 1; i < a.length; i++)
            if (a[i] < a[i-1])
                return false;
        return true;
    }

    public static Map<Integer,Integer> getFrequency(int[] a) {
        Map<Integer,Integer> m = new HashMap<Integer,Integer>();
        for (int i = 0; i < a.length; i++) {
            if (m.containsKey(a[i]))
                m.put(a[i], m.get(a[i]) + 1);
            else
                m.put(a[i], 1);
        }
        return m;
    }

    static String display(int[] before, int[] after) {
        return "\n  " + Arrays.toString(before) +
            " -> " + Arrays.toString(after);
    }

    static void testSort(Sort s, int[] a) {
        int[] copy = Arrays.copyOf(a, a.length);
        Map<Integer,Integer> m1 = getFrequency(a);
        try {
            s.sort(a);
        } catch(Exception e) {
            fail("  raised exception " + e.toString());
        }
        if (!checkOrdered(a))
            fail("  not sorted: " + Arrays.toString(a));
        Map<Integer,Integer> m2 = getFrequency(a);
        if (!m2.equals(m1))
            fail("  elements have changed" + display(copy, a));
    }

    /* un autre façon de procéder, en comparant avec le résultat de Arrays.sort */
    static void testSort2(Sort s, int[] a) {
        int[] copy = Arrays.copyOf(a, a.length);
        Arrays.sort(copy);
        try {
            s.sort(a);
        } catch(Exception e) {
            fail("  raised exception " + e);
        }
        if(!Arrays.equals(a, copy))
        	fail("  final array is " + Arrays.toString(a) + " but should be " + Arrays.toString(copy));
    }
 
    static void testSort(Sort s) {
        int[][] arrays = { {},
                           {42},
                           {2,1,1},
                           {10,9,8,7,6,5,4,3,2,1},
                           {1,2,3,4,5,6,7,8,9,10},
                           {1,2,3,4,5,1,2,3,4,5},
                           {0,-1,0,1,2,-2}
        };
        try {
            for (int[] a: arrays)
                testSort(s, a);
            System.out.println(s.getClass() + " seems correct!");
        } catch (SortError e) {
            System.err.println(s.getClass() + " is incorrect!");
        }
    }
    
    public static void main(String[] args) {
    	testSort(new Sort1());
    	testSort(new Sort2());
    	testSort(new Sort3());
    	testSort(new Sort4());
    	testSort(new Sort5());
    	testSort(new Sort6());
    	testSort(new Sort7());
    	testSort(new Sort8());
    	testSort(new Sort9());
    	testSort(new Sort10());
    	testSort(new Sort11());
    	testSort(new Sort12());
    	testSort(new Sort13());
    	testSort(new Sort14());
    }
    
    /* note : on peut éviter ce copier-coller de la manière suivante */
//    public static void main(String[] args) throws ClassNotFoundException, InstantiationException, IllegalAccessException {
//    	for (int i = 1; i < 15; i++) {
//    		@SuppressWarnings("unchecked")
//			Class<Sort> c = (Class<Sort>) Class.forName("intsort.Sort" + i);
//    		testSort(c.newInstance());
//    	}
//    }
}