package intfifo;


public class ArrayQueue3 implements FIFO {
	
	private int maxsize; 
	private int [] fifo;
	private int size;
	private int older; // les éléments sont dans older..older+size-1
	
	public ArrayQueue3 (int max) {
		this.maxsize = max;
		this.fifo = new int [maxsize] ;
		this.size = 0;
		this.older = 0;
	}
	
	public int size () {return this.size ;}
	
	@Override
	public void put(int n) throws FullQueueException {
		this.size++ ;
		int younger = (older + size - 1) % maxsize ;
		fifo[younger] = n ;
	}

	@Override
	public int get() throws EmptyQueueException {
		if (this.size () == 0) throw new EmptyQueueException () ;
		this.size-- ;
		int x = fifo[older] ;
		older = ((older + 1) % maxsize) ;
		return x ;
	}

	@Override
	public void flush() {
		size = 0 ;
	}

	@Override
	public void print() {
		int s = this.size () ;
		int current = older ;
		for (int i = 0 ; i < s ; i++) {
			System.out.print(fifo[current]+" ") ;
			current = (current + 1) % maxsize ;
		}
	}
	
	@Override
	public boolean is_empty () {
		return (this.size == 0) ;
	}

	/* Le code ci-dessous est à utiliser uniquement
	 * si on souhaite redimensionner le tableau
	 */
	
	/*
	private int [] concat(int [] a, int [] b) {
		   int aLen = a.length ;
		   int bLen = b.length ;
		   int [] c = new int [aLen+bLen] ;
		   System.arraycopy(a, 0, c, 0, aLen) ;
		   System.arraycopy(b, 0, c, aLen, bLen) ;
		   return c ;
		}
	*/

	public int capacity () {
		return maxsize ;
	}
	
	/*
	private void transfer (int [] dest) {
		if (this.size () <= dest.length) {
			for(int i = 0 ; i < this.size ; i++) {
				dest[i] = this.fifo[(older + i) % this.fifo.length] ;
			} ;
			this.fifo = dest ;
		}
	}
	*/
	
	public int move (int n) {
		if (n >= this.size ()) {
			int [] dest = new int [n] ;
			int wrap = this.older + this.size - this.fifo.length ;
			if (wrap <= 0)
				System.arraycopy(this.fifo, this.older, dest, 0, this.size) ;
			else { // wraps
				System.arraycopy(this.fifo, this.older, dest, 0, this.size - wrap) ;
				System.arraycopy(this.fifo, 0, dest, this.size - wrap, wrap) ;
			}
			this.older = 0 ;
			this.fifo = dest ;
			this.maxsize = n ; 
		} ; 
		return maxsize ;
	}
	
	public int increase_capacity (int n) {
		return this.move(this.maxsize + n) ;
	}

	public int decrease_capacity (int n) {
		return this.move(this.maxsize - n) ;
	}

	/*
	public int decrease_capacity (int n) {
		int reduced_maxsize = maxsize - n ;
		if (reduced_maxsize >= this.size ()) {
			int [] fifo = new int [reduced_maxsize];
			int wrap = this.older + this.size - this.fifo.length;
			if (wrap <= 0)
				System.arraycopy(this.fifo, this.older, fifo, 0, this.size) ;
			else { // wraps
				System.arraycopy(this.fifo, this.older, fifo, 0, this.size - wrap);
				System.arraycopy(this.fifo, 0, fifo, this.size - wrap, wrap);
			}
			this.older = 0;
			this.fifo = fifo ;
			maxsize = reduced_maxsize ;
		}
		return maxsize ;
	}
	*/

}
